{% extends "admin/base_site.html" %}

{% block content %}
<form action="{{ form_url }}" method="post" id="lexicon_form" class="pylucid_form">
    {% include "admin/pylucid/includes/form_fieldset_errorlist.html" %}
    {% include "admin/pylucid/includes/markup_preview_fieldset.html" %}
    <fieldset>
        {% include "admin/pylucid/includes/form_default_buttons.html" %}
        {% include "admin/pylucid/includes/pylucid_formset.html" %}
        {% include "admin/pylucid/includes/form_default_buttons.html" %}
    </fieldset>
</form>
{% endblock %}
