{% extends "page_admin/markup_help_base.html" %}

{% block content %}{{ block.super }}
<img alt="creole cheat sheet" class="float_left" src="{{ PyLucid_media_url }}markup_help/creole_cheat_sheet.png">
<h2>links</h2>
<ul>
	<li><a class="add_openinwindow" href="http://wikicreole.org/wiki/Creole1.0">Creole v1.0 spec</a></li>
	<li><a class="add_openinwindow" href="http://wiki.wikicreole.org/">wiki.wikicreole.org</a></li>
</ul>
{% endblock content %}