{% extends "pylucid/css_anchor_div.html" %}

{% block plugin_content %}
{% if update_list %}
<table id="page_updates">
  <tr>
    <th>no</th>
    <th>update time</th>
    <th>type</th>
    <th title="content language">language</th>
    <th>title</th>
    <th>user name</th>
    {% if perms.update_journal.change_updatejournal or perms.update_journal.delete_updatejournal %}
        <th>actions</th>
    {% endif %}
  </tr>
  {% for item in update_list %}
  <tr class="{% cycle 'row1' 'row2' %}">
    <td>{{ forloop.counter }}</td>
    <td>{{ item.lastupdatetime|date:_("DATETIME_FORMAT") }}</td>
    <td>{{ item.content_type }}</td>
    <td title="{{ item.language.description }}">{{ item.language.code }}</td>
    <td><a href="{{ item.object_url }}">{{ item.title|escape }}</a></td>
    <td>{{ item.user_name }}</td>
    
    {% if perms.update_journal.change_updatejournal or perms.update_journal.delete_updatejournal %}
    <td>
        {% if perms.update_journal.change_updatejournal %}
            <a href="{% url admin:update_journal_updatejournal_change item.id %}" title="edit '{{ item.title|escape }}'">edit</a>
        {% endif %}
        {% if perms.update_journal.delete_updatejournal %}
            <a href="{% url admin:update_journal_updatejournal_delete item.id %}" title="Delete '{{ item.title|escape }}' from the list">delete</a>
        {% endif %}
    </td>
    {% endif %}
  </tr>
  {% endfor %}
</table>
{% if select_feed_url %}
<a href="{{ select_feed_url }}" class="select_feeds" title="{% trans "select update journal feed format" %}">
	{% trans "get update journal as RSS/Atom feed." %}
</a>
{% endif %}
{% else %}
    <p>No updates recorded in update journal.</p>
{% endif %}
{% endblock %}