<div class="helper_buttons">
    <input type="submit" name="save" value="{% trans 'save' %}" />
    <input onclick="self.location.href='{{ abort_url }}'" name="abort" value="{% trans 'abort' %}" type="reset" />
    {% if help_link %}
        <a href="{{ help_link }}" class="openinwindow" title="{% trans 'markup help page' %}">
            <button type="button">{% trans 'markup help' %}</button>
        </a>
    {% endif %}
    <input type="submit" id="preview" name="preview" value="{% trans 'markup preview' %}" />
    <a href="{% url PageAdmin-page_list %}?markup={{ pagecontent.markup }}" class="openinwindow" title="{% trans 'List of all pages for creating links' %}">
        <button type="button" class="button">{% trans 'page list' %}</button>
    </a>
    <a href="{% url PageAdmin-tag_list %}" class="openinwindow" title="{% trans 'List of all available PyLucid template tags' %}">
        <button type="button" class="button">{% trans 'tag list' %}</button>
    </a>

     {% if markup_id_str in "26" %}
     <a href="{% url PageAdmin-markup_help %}?markup={{ pagecontent.markup }}" class="openinwindow" title="{% trans 'Markup help' %}">
         <button type="button" class="button">{% trans 'markup help' %}</button>
     </a>
     {% endif %}

    <button type="button" class="button resize_textarea" id="bigger_id_content" title="{% trans 'makes the textarea bigger' %}">
        +
    </button>
    <button type="button" class="button resize_textarea" id="smaller_id_content" title="{% trans 'makes the textarea smaller' %}">
        -
    </button>
 </div>