{% extends template_name %}
{% load comments %}

{% block content %}
<div class="{{ CSS_PLUGIN_CLASS_NAME }} blog">
{% include "blog/includes/tagcloud.html" %}

{% if not entries %}no blog entry exists{% endif %}
{% for entry in entries.object_list %}
<fieldset class="entry">
	{% include "blog/includes/blog_entry.html" %}
	{% get_comment_count for entry as comment_count %}            
	<a href="{{ entry.get_absolute_url }}" class="comment_link" title="Display all comments | Leave a comment">{{ comment_count }} comment{{ comment_count|pluralize:"s" }}...</a>
</fieldset>
{% endfor %}
</div>

<fieldset class="feed_selection"><legend>{% trans "Syndication feed format:" %}</legend>
<ul>
{% for feed in feeds %}
    <li>
        {% if tags %}
            <a href="{% url Blog-tag_feed tags feed.filename %}">{% url Blog-tag_feed tags feed.filename %}</a>
        {% else %}      
            <a href="{% url Blog-feed feed.filename %}">{% url Blog-feed feed.filename %}</a>
        {% endif %}
        <small>(
        {% if feed.filename == "feed.rss" %}
            <a href="{% trans "http://en.wikipedia.org/wiki/RSS" %}">Really Simple Syndication</a> v2.01rev2
        {% endif %}
        {% if feed.filename == "feed.atom" %}   
            <a href="{% trans "http://en.wikipedia.org/wiki/Atom_(standard)" %}">Atom Syndication Format</a> v1.0
        {% endif %}
        )
        </small>
    </li>    
{% endfor %}
</lu>
</fieldset>

<div class="pagination">
    {% if entries.has_previous %}
        <a href="?page={{ entries.previous_page_number }}" title="{% trans "&lt; goto previous page" %}" class="previous">&#x22D8; {% trans "previous" %}</a>
    {% endif %}
    
    <span class="pagelist">
    {% for entry in entries.paginator.page_range %}
        {% if entry == entries.number %}
            <strong>{{ entry }}</strong>
        {% else %}
            <a href="?page={{ entry }}">{{ entry }}</a>
        {% endif %}
    {% endfor %}
    </span>
 
    {% if entries.has_next %}
        <a href="?page={{ entries.next_page_number }}" title="{% trans "goto next page &gt;" %}" class="next">{% trans "next" %} &#x22D9;</a>
    {% endif %}
</div>
{% endblock content %}