{% extends "admin/base_site.html" %}
{% load i18n %}

{% block extrastyle %}{{ block.super }}
<!-- pylucid admin menu extrahead block start -->
<link rel="stylesheet" type="text/css" media="screen" href="{{ PyLucid_media_url }}superfish/superfish-vertical.css" />
<!-- pylucid admin menu extrahead block end -->
{% endblock %}

{% block content %}
<p>Please select a sub menu point.</p>
<ul>
	{% lucidTag admin_menu.panel_extras %}
</ul>
{% endblock %}