{% extends "pylucid_admin/base_site.html" %}
{% load i18n %}

{% block extrahead %}{{ block.super }}
<!-- internals.form_generator extrahead block start -->
<style type="text/css">
/* ----------------------------------------------------------------------
   TODO: This should be stored into a Editable Headfile!
---------------------------------------------------------------------- */
.pygments_code, .pygments_code table, .pygments_code .code, .pygments_code pre {
  background: #ffffff;
  padding: 0;
  margin: 0;
}
.pygments_code pre {
 border: 1px;
}
.pygments_code .code {
  width: 100%;
}
.pygments  { background: #ffffff; }
.pygments .c { color: #808080 } /* Comment */
.pygments .err { color: #F00000; background-color: #F0A0A0 } /* Error */
.pygments .k { color: #008000; font-weight: bold } /* Keyword */
.pygments .o { color: #303030 } /* Operator */
.pygments .cm { color: #808080 } /* Comment.Multiline */
.pygments .cp { color: #507090 } /* Comment.Preproc */
.pygments .c1 { color: #808080 } /* Comment.Single */
.pygments .cs { color: #cc0000; font-weight: bold } /* Comment.Special */
.pygments .gd { color: #A00000 } /* Generic.Deleted */
.pygments .ge { font-style: italic } /* Generic.Emph */
.pygments .gr { color: #FF0000 } /* Generic.Error */
.pygments .gh { color: #000080; font-weight: bold } /* Generic.Heading */
.pygments .gi { color: #00A000 } /* Generic.Inserted */
.pygments .go { color: #808080 } /* Generic.Output */
.pygments .gp { color: #c65d09; font-weight: bold } /* Generic.Prompt */
.pygments .gs { font-weight: bold } /* Generic.Strong */
.pygments .gu { color: #800080; font-weight: bold } /* Generic.Subheading */
.pygments .gt { color: #0040D0 } /* Generic.Traceback */
.pygments .kc { color: #008000; font-weight: bold } /* Keyword.Constant */
.pygments .kd { color: #008000; font-weight: bold } /* Keyword.Declaration */
.pygments .kp { color: #003080; font-weight: bold } /* Keyword.Pseudo */
.pygments .kr { color: #008000; font-weight: bold } /* Keyword.Reserved */
.pygments .kt { color: #303090; font-weight: bold } /* Keyword.Type */
.pygments .m { color: #6000E0; font-weight: bold } /* Literal.Number */
.pygments .s { background-color: #fff0f0 } /* Literal.String */
.pygments .na { color: #0000C0 } /* Name.Attribute */
.pygments .nb { color: #007020 } /* Name.Builtin */
.pygments .nc { color: #B00060; font-weight: bold } /* Name.Class */
.pygments .no { color: #003060; font-weight: bold } /* Name.Constant */
.pygments .nd { color: #505050; font-weight: bold } /* Name.Decorator */
.pygments .ni { color: #800000; font-weight: bold } /* Name.Entity */
.pygments .ne { color: #F00000; font-weight: bold } /* Name.Exception */
.pygments .nf { color: #0060B0; font-weight: bold } /* Name.Function */
.pygments .nl { color: #907000; font-weight: bold } /* Name.Label */
.pygments .nn { color: #0e84b5; font-weight: bold } /* Name.Namespace */
.pygments .nt { color: #007000 } /* Name.Tag */
.pygments .nv { color: #906030 } /* Name.Variable */
.pygments .ow { color: #000000; font-weight: bold } /* Operator.Word */
.pygments .w { color: #bbbbbb } /* Text.Whitespace */
.pygments .mf { color: #6000E0; font-weight: bold } /* Literal.Number.Float */
.pygments .mh { color: #005080; font-weight: bold } /* Literal.Number.Hex */
.pygments .mi { color: #0000D0; font-weight: bold } /* Literal.Number.Integer */
.pygments .mo { color: #4000E0; font-weight: bold } /* Literal.Number.Oct */
.pygments .sb { background-color: #fff0f0 } /* Literal.String.Backtick */
.pygments .sc { color: #0040D0 } /* Literal.String.Char */
.pygments .sd { color: #D04020 } /* Literal.String.Doc */
.pygments .s2 { background-color: #fff0f0 } /* Literal.String.Double */
.pygments .se { color: #606060; font-weight: bold; background-color: #fff0f0 } /* Literal.String.Escape */
.pygments .sh { background-color: #fff0f0 } /* Literal.String.Heredoc */
.pygments .si { background-color: #e0e0e0 } /* Literal.String.Interpol */
.pygments .sx { color: #D02000; background-color: #fff0f0 } /* Literal.String.Other */
.pygments .sr { color: #000000; background-color: #fff0ff } /* Literal.String.Regex */
.pygments .s1 { background-color: #fff0f0 } /* Literal.String.Single */
.pygments .ss { color: #A06000 } /* Literal.String.Symbol */
.pygments .bp { color: #007020 } /* Name.Builtin.Pseudo */
.pygments .vc { color: #306090 } /* Name.Variable.Class */
.pygments .vg { color: #d07000; font-weight: bold } /* Name.Variable.Global */
.pygments .vi { color: #3030B0 } /* Name.Variable.Instance */
.pygments .il { color: #0000D0; font-weight: bold } /* Literal.Number.Integer.Long */
</style>
<!-- internals.form_generator extrahead block end -->
{% endblock %}

{% block content %}
<!-- view_content block -->
<p>Simple form from model generator.
Based on <a href="http://code.djangoproject.com/attachment/wiki/FormGenScript/form_generator.py">form_generator.py</a>,
see also <a href="http://code.djangoproject.com/wiki/FormGenScript">code.djangoproject.com/wiki/FormGenScript</a>.</p>
<ul>
	{% for no,model in models_dict.items %}
	<li><a href="{% url Internal-form_generator model_no=no %}">{{ model }}</a></li>
	{% endfor %}
</ul>
{% if output %}{{ output }}
{% else %}
    <p>Please select.</p>
{% endif %}
<!-- view_content block end -->
{% endblock %}