{% extends template_name %}
{% load comments %}

{% block content %}
<div class="{{ CSS_PLUGIN_CLASS_NAME }} blog">
{% include "blog/includes/tagcloud.html" %}

<fieldset class="entry">
    {% include "blog/includes/blog_entry.html" %}
    {% include "pylucid_comments/includes/comments_list.html" %}
</fieldset>
</div>
{% endblock content %}